@@ -174,7 +174,7 @@ vec_rbind <- function(...,
                       .names_to = rlang::zap(),
                       .name_repair = c("unique", "universal", "check_unique"),
                       .name_spec = NULL) {
-  .External2(vctrs_rbind, .ptype, .names_to, .name_repair, .name_spec)
+  .External2(ffi_rbind, .ptype, .names_to, .name_repair, .name_spec)
 }
 vec_rbind <- fn_inline_formals(vec_rbind, ".name_repair")
 
@@ -189,29 +189,29 @@ vec_cbind <- function(...,
                       .ptype = NULL,
                       .size = NULL,
                       .name_repair = c("unique", "universal", "check_unique", "minimal")) {
-  .External2(vctrs_cbind, .ptype, .size, .name_repair)
+  .External2(ffi_cbind, .ptype, .size, .name_repair)
 }
 vec_cbind <- fn_inline_formals(vec_cbind, ".name_repair")
 
 as_df_row <- function(x, quiet = FALSE) {
-  .Call(vctrs_as_df_row, x, quiet)
+  .Call(ffi_as_df_row, x, quiet, environment())
 }
 as_df_col <- function(x, outer_name) {
-  .Call(vctrs_as_df_col, x, outer_name)
+  .Call(ffi_as_df_col, x, outer_name, environment())
 }
 
 #' Frame prototype
 #'
 #' @description
 #'
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("experimental")}
+#' `r lifecycle::badge("experimental")`
 #'
 #' This is an experimental generic that returns zero-columns variants
 #' of a data frame. It is needed for [vec_cbind()], to work around the
 #' lack of colwise primitives in vctrs. Expect changes.
 #'
 #' @param x A data frame.
-#' @inheritParams ellipsis::dots_empty
+#' @inheritParams rlang::args_dots_empty
 #'
 #' @keywords internal
 #' @export
