@@ -32,6 +32,9 @@
 #' levels are nested.
 #'
 #'
+#' @inheritParams rlang::args_error_context
+#' @inheritParams rlang::args_dots_empty
+#'
 #' @param names A character vector.
 #' @param repair Either a string or a function. If a string, it must
 #'   be one of `"check_unique"`, `"minimal"`, `"unique"`, or `"universal"`.
@@ -57,7 +60,9 @@
 #'   caused by repairing the names. This only concerns unique and
 #'   universal repairing. Set `quiet` to `TRUE` to silence the
 #'   messages.
-#' @inheritParams ellipsis::dots_empty
+#'
+#'   Users can silence the name repair messages by setting the
+#'   `"rlib_name_repair_verbosity"` global option to `"quiet"`.
 #'
 #' @section `minimal` names:
 #'
@@ -154,34 +159,43 @@ vec_as_names <- function(names,
                          ...,
                          repair = c("minimal", "unique", "universal", "check_unique"),
                          repair_arg = "",
-                         quiet = FALSE) {
-  if (!missing(...)) {
-    ellipsis::check_dots_empty()
-  }
-  .Call(vctrs_as_names, names, repair, repair_arg, quiet)
+                         quiet = FALSE,
+                         call = caller_env()) {
+  check_dots_empty0(...)
+  .Call(
+    ffi_as_names,
+    names,
+    repair,
+    quiet,
+    environment()
+  )
 }
 
+# TODO! Error calls
 validate_name_repair_arg <- function(repair) {
   .Call(vctrs_validate_name_repair_arg, repair)
 }
 validate_minimal_names <- function(names, n = NULL) {
   .Call(vctrs_validate_minimal_names, names, n)
 }
-validate_unique <- function(names, arg = "", n = NULL) {
+validate_unique <- function(names,
+                            arg = "",
+                            n = NULL,
+                            call = caller_env()) {
   validate_minimal_names(names, n)
 
   empty_names <- detect_empty_names(names)
   if (has_length(empty_names)) {
-    stop_names_cannot_be_empty(names)
+    stop_names_cannot_be_empty(names, call = call)
   }
 
   dot_dot_name <- detect_dot_dot(names)
   if (has_length(dot_dot_name)) {
-    stop_names_cannot_be_dot_dot(names)
+    stop_names_cannot_be_dot_dot(names, call = call)
   }
 
   if (anyDuplicated(names)) {
-    stop_names_must_be_unique(names, arg)
+    stop_names_must_be_unique(names, arg, call = call)
   }
 
   invisible(names)
@@ -242,9 +256,7 @@ vec_names2 <- function(x,
                        ...,
                        repair = c("minimal", "unique", "universal", "check_unique"),
                        quiet = FALSE) {
-  if (!missing(...)) {
-    ellipsis::check_dots_empty()
-  }
+  check_dots_empty0(...)
   repair <- validate_name_repair_arg(repair)
 
   if (is_function(repair)) {
@@ -258,7 +270,8 @@ vec_names2 <- function(x,
     return(new_names)
   }
 
-  switch(repair,
+  switch(
+    repair,
     minimal = minimal_names(x),
     unique = unique_names(x, quiet = quiet),
     universal = as_universal_names(minimal_names(x), quiet = quiet),
@@ -404,25 +417,7 @@ re_match <- function(text, pattern, perl = TRUE, ...) {
 
 
 describe_repair <- function(orig_names, names) {
-  if (is_null(orig_names)) {
-    orig_names <- rep_along(names, "")
-  }
-  if (length(orig_names) != length(names)) {
-    stop("Internal error: New names and old names don't have same length")
-  }
-
-  new_names <- names != as_minimal_names(orig_names)
-  if (any(new_names)) {
-    msg <- bullets(
-      header = "New names:",
-      paste0(
-        tick_if_needed(orig_names[new_names]),
-        " -> ",
-        tick_if_needed(names[new_names])
-      )
-    )
-    message(msg)
-  }
+  names_inform_repair(orig_names, names)
 }
 
 bullets <- function(..., header = NULL) {
@@ -443,30 +438,14 @@ bullets <- function(..., header = NULL) {
   info
 }
 
-tick <- function(x) {
-  ifelse(is.na(x), "NA", encodeString(x, quote = "`"))
-}
-
-is_syntactic <- function(x) {
-  ret <- (make_syntactic(x) == x)
-  ret[is.na(x)] <- FALSE
-  ret
-}
-
-tick_if_needed <- function(x) {
-  needs_ticks <- !is_syntactic(x)
-  x[needs_ticks] <- tick(x[needs_ticks])
-  x
-}
-
 # Used in names.c
-set_rownames_fallback <- function(x, names) {
+set_rownames_dispatch <- function(x, names) {
   rownames(x) <- names
   x
 }
 
 # Used in names.c
-set_names_fallback <- function(x, names) {
+set_names_dispatch <- function(x, names) {
   names(x) <- names
   x
 }
