@@ -27,7 +27,7 @@
 #' @examples
 #' new_data_frame(list(x = 1:10, y = 10:1))
 new_data_frame <- function(x = list(), n = NULL, ..., class = NULL) {
-  .External(vctrs_new_data_frame, x, n, class, ...)
+  .External(ffi_new_data_frame, x, n, class, ...)
 }
 new_data_frame <- fn_inline_formals(new_data_frame, "x")
 
@@ -84,7 +84,7 @@ new_data_frame <- fn_inline_formals(new_data_frame, "x")
 df_list <- function(...,
                     .size = NULL,
                     .name_repair = c("check_unique", "unique", "universal", "minimal")) {
-  .Call(vctrs_df_list, list2(...), .size, .name_repair)
+  .Call(ffi_df_list, list2(...), .size, .name_repair, environment())
 }
 df_list <- fn_inline_formals(df_list, ".name_repair")
 
@@ -155,7 +155,7 @@ df_list <- fn_inline_formals(df_list, ".name_repair")
 data_frame <- function(...,
                        .size = NULL,
                        .name_repair = c("check_unique", "unique", "universal", "minimal")) {
-  .Call(vctrs_data_frame, list2(...), .size, .name_repair)
+  .Call(ffi_data_frame, list2(...), .size, .name_repair, environment())
 }
 data_frame <- fn_inline_formals(data_frame, ".name_repair")
 
@@ -184,11 +184,10 @@ vec_ptype_full.data.frame <- function(x, ...) {
 #' @export
 vec_ptype_abbr.data.frame <- function(x, ...) {
   if (inherits_only(x, "data.frame")) {
-    abbr <- "df"
+    "df"
   } else {
-    abbr <- class(x)[[1]]
+    class(x)[[1]]
   }
-  paste0(abbr, vec_ptype_shape(x))
 }
 
 #' @export
@@ -251,17 +250,45 @@ df_is_coercible <- function(x, y, opts) {
 #'   as a bare tibble.
 #'
 #' @export
-df_ptype2 <- function(x, y, ..., x_arg = "", y_arg = "") {
-  .Call(vctrs_df_ptype2_opts, x, y, opts = match_fallback_opts(...), x_arg, y_arg)
+df_ptype2 <- function(x,
+                      y,
+                      ...,
+                      x_arg = "",
+                      y_arg = "",
+                      call = caller_env()) {
+  .Call(
+    ffi_df_ptype2_opts,
+    x,
+    y,
+    opts = match_fallback_opts(...),
+    environment()
+  )
 }
 #' @rdname df_ptype2
 #' @export
-df_cast <- function(x, to, ..., x_arg = "", to_arg = "") {
-  .Call(vctrs_df_cast_opts, x, to, opts = match_fallback_opts(...), x_arg, to_arg)
+df_cast <- function(x,
+                    to,
+                    ...,
+                    x_arg = "",
+                    to_arg = "",
+                    call = caller_env()) {
+  .Call(
+    ffi_df_cast_opts,
+    x,
+    to,
+    opts = match_fallback_opts(...),
+    environment()
+  )
 }
 
-df_ptype2_opts <- function(x, y, ..., opts, x_arg = "", y_arg = "") {
-  .Call(vctrs_df_ptype2_opts, x, y, opts = opts, x_arg, y_arg)
+df_ptype2_opts <- function(x,
+                           y,
+                           ...,
+                           opts,
+                           x_arg = "",
+                           y_arg = "",
+                           call = caller_env()) {
+  .Call(ffi_df_ptype2_opts, x, y, opts = opts, environment())
 }
 
 df_cast_opts <- function(x,
@@ -269,8 +296,15 @@ df_cast_opts <- function(x,
                          ...,
                          opts = fallback_opts(),
                          x_arg = "",
-                         to_arg = "") {
-  .Call(vctrs_df_cast_opts, x, to, opts, x_arg, to_arg)
+                         to_arg = "",
+                         call = caller_env()) {
+  .Call(
+    ffi_df_cast_opts,
+    x,
+    to,
+    opts,
+    environment()
+  )
 }
 df_cast_params <- function(x,
                            to,
@@ -308,11 +342,11 @@ vec_ptype2.data.frame.data.frame <- function(x, y, ...) {
   df_ptype2(x, y, ...)
 }
 
-vec_ptype2_df_fallback_normalise <- function(x, y, opts) {
+vec_ptype2_df_fallback_normalise <- function(x, y, opts, ...) {
   x_orig <- x
   y_orig <- y
 
-  ptype <- df_ptype2_opts(x, y, opts = opts)
+  ptype <- df_ptype2_opts(x, y, opts = opts, ...)
 
   x <- x[0, , drop = FALSE]
   y <- y[0, , drop = FALSE]
@@ -334,9 +368,9 @@ vec_ptype2_df_fallback_normalise <- function(x, y, opts) {
 
   list(x = x, y = y)
 }
-vec_cast_df_fallback_normalise <- function(x, to, opts) {
+vec_cast_df_fallback_normalise <- function(x, to, opts, ...) {
   orig <- x
-  cast <- df_cast_opts(x, to, opts = opts)
+  cast <- df_cast_opts(x, to, opts = opts, ...)
 
   # Seq-assign should be more widely implemented than empty-assign?
   x[seq_along(to)] <- cast
@@ -357,14 +391,20 @@ df_needs_normalisation <- function(x, y, opts) {
 }
 
 # Fallback for data frame subclasses (#981)
-vec_ptype2_df_fallback <- function(x, y, opts, x_arg = "", y_arg = "") {
+vec_ptype2_df_fallback <- function(x,
+                                   y,
+                                   opts,
+                                   x_arg = "",
+                                   y_arg = "",
+                                   call = caller_env()) {
   seen_tibble <- inherits(x, "tbl_df") || inherits(y, "tbl_df")
 
   ptype <- vec_ptype2_params(
     new_data_frame(x),
     new_data_frame(y),
     df_fallback = opts$df_fallback,
-    s3_fallback = opts$s3_fallback
+    s3_fallback = opts$s3_fallback,
+    call = call
   )
 
   classes <- NULL
@@ -456,7 +496,7 @@ vec_cast.data.frame <- function(x, to, ...) {
 #' @export
 #' @method vec_cast.data.frame data.frame
 vec_cast.data.frame.data.frame <- function(x, to, ..., x_arg = "", to_arg = "") {
-  df_cast(x, to, x_arg = x_arg, to_arg = to_arg)
+  df_cast(x, to, ..., x_arg = x_arg, to_arg = to_arg)
 }
 
 #' @export
@@ -470,7 +510,13 @@ df_size <- function(x) {
   .Call(vctrs_df_size, x)
 }
 
-df_lossy_cast <- function(out, x, to, ..., x_arg = "", to_arg = "") {
+df_lossy_cast <- function(out,
+                          x,
+                          to,
+                          ...,
+                          x_arg = "",
+                          to_arg = "",
+                          call = caller_env()) {
   extra <- setdiff(names(x), names(to))
 
   maybe_lossy_cast(
@@ -481,6 +527,7 @@ df_lossy_cast <- function(out, x, to, ..., x_arg = "", to_arg = "") {
     locations = int(),
     x_arg = x_arg,
     to_arg = to_arg,
+    call = call,
     details = inline_list("Dropped variables: ", extra, quote = "`"),
     class = "vctrs_error_cast_lossy_dropped"
   )
