@@ -15,8 +15,11 @@ SEXP vctrs_c(SEXP call, SEXP op, SEXP args, SEXP env) {
   SEXP name_spec = PROTECT(Rf_eval(CAR(args), env)); args = CDR(args);
   SEXP name_repair = PROTECT(Rf_eval(CAR(args), env));
 
-  struct name_repair_opts name_repair_opts = new_name_repair_opts(name_repair, args_empty, false);
-  PROTECT_NAME_REPAIR_OPTS(&name_repair_opts);
+  struct name_repair_opts name_repair_opts = new_name_repair_opts(name_repair,
+                                                                  vec_args.empty,
+                                                                  false,
+                                                                  r_lazy_null);
+  KEEP(name_repair_opts.shelter);
 
   SEXP out = vec_c(xs, ptype, name_spec, &name_repair_opts);
 
@@ -42,8 +45,12 @@ SEXP vec_c_opts(SEXP xs,
                 SEXP name_spec,
                 const struct name_repair_opts* name_repair,
                 const struct fallback_opts* fallback_opts) {
+  struct ptype_common_opts ptype_opts = {
+    .fallback = *fallback_opts
+  };
+
   SEXP orig_ptype = ptype;
-  ptype = PROTECT(vec_ptype_common_opts(xs, orig_ptype, fallback_opts));
+  ptype = PROTECT(vec_ptype_common_opts(xs, orig_ptype, &ptype_opts));
 
   if (ptype == R_NilValue) {
     UNPROTECT(1);
@@ -73,11 +80,10 @@ SEXP vec_c_opts(SEXP xs,
   // fallback disabled as well.
   if ((is_data_frame(ptype) && fallback_opts->s3 == S3_FALLBACK_true) ||
       vec_is_common_class_fallback(ptype)) {
-    struct fallback_opts d_fallback_opts = *fallback_opts;
-    d_fallback_opts.s3 = S3_FALLBACK_false;
-    ptype = PROTECT(vec_ptype_common_opts(xs, orig_ptype, &d_fallback_opts));
+    ptype_opts.fallback.s3 = S3_FALLBACK_false;
+    ptype = PROTECT(vec_ptype_common_opts(xs, orig_ptype, &ptype_opts));
   } else {
-    ptype = PROTECT(vec_ptype_common_opts(xs, ptype, fallback_opts));
+    ptype = PROTECT(vec_ptype_common_opts(xs, ptype, &ptype_opts));
   }
 
   // Find individual input sizes and total size of output
@@ -122,24 +128,11 @@ SEXP vec_c_opts(SEXP xs,
   };
 
   for (R_len_t i = 0; i < n; ++i) {
+    SEXP x = VECTOR_ELT(xs, i);
     R_len_t size = p_sizes[i];
-    if (!size) {
-      continue;
-    }
-
-    struct cast_opts opts = (struct cast_opts) {
-      .x = VECTOR_ELT(xs, i),
-      .to = ptype,
-      .fallback = *fallback_opts
-    };
-    SEXP x = PROTECT(vec_cast_opts(&opts));
 
     init_compact_seq(p_loc, counter, size, true);
 
-    // Total ownership of `out` because it was freshly created with `vec_init()`
-    out = vec_proxy_assign_opts(out, loc, x, VCTRS_OWNED_true, &c_assign_opts);
-    REPROTECT(out, out_pi);
-
     if (assign_names) {
       SEXP outer = xs_is_named ? STRING_ELT(xs_names, i) : R_NilValue;
       SEXP inner = PROTECT(vec_names(x));
@@ -159,6 +152,21 @@ SEXP vec_c_opts(SEXP xs,
       UNPROTECT(2);
     }
 
+    if (!size) {
+      continue;
+    }
+
+    struct cast_opts opts = (struct cast_opts) {
+      .x = x,
+      .to = ptype,
+      .fallback = *fallback_opts
+    };
+    x = PROTECT(vec_cast_opts(&opts));
+
+    // Total ownership of `out` because it was freshly created with `vec_init()`
+    out = vec_proxy_assign_opts(out, loc, x, VCTRS_OWNED_true, &c_assign_opts);
+    REPROTECT(out, out_pi);
+
     counter += size;
     UNPROTECT(1);
   }
@@ -273,19 +281,21 @@ SEXP vec_c_fallback(SEXP ptype,
   if (implements_c) {
     return vec_c_fallback_invoke(xs, name_spec);
   } else {
-    struct fallback_opts fallback_opts = {
-      .df = DF_FALLBACK_none,
-      .s3 = S3_FALLBACK_false
+    struct ptype_common_opts ptype_opts = {
+      .fallback = {
+        .df = DF_FALLBACK_none,
+        .s3 = S3_FALLBACK_false
+      }
     };
 
     // Should cause a common type error, unless another fallback
     // kicks in (for instance, homogeneous class with homogeneous
     // attributes)
-    vec_ptype_common_opts(xs, R_NilValue, &fallback_opts);
+    vec_ptype_common_opts(xs, R_NilValue, &ptype_opts);
 
     // Suboptimal: Call `vec_c()` again to combine vector with
     // homogeneous class fallback
-    return vec_c_opts(xs, R_NilValue, name_spec, name_repair, &fallback_opts);
+    return vec_c_opts(xs, R_NilValue, name_spec, name_repair, &ptype_opts.fallback);
   }
 }
 
