@@ -167,7 +167,7 @@ test_that("can determine equality of strings with different encodings (#553)", {
 })
 
 test_that("equality can be determined when strings have identical encodings", {
-  encs <- encodings(bytes = TRUE)
+  encs <- encodings()
 
   for (enc in encs) {
     expect_true(vec_equal(enc, enc))
@@ -175,6 +175,12 @@ test_that("equality can be determined when strings have identical encodings", {
   }
 })
 
+test_that("equality is known to always fail with bytes", {
+  enc <- encoding_bytes()
+  error <- "translating strings with \"bytes\" encoding"
+  expect_error(vec_equal(enc, enc), error)
+})
+
 test_that("equality is known to fail when comparing bytes to other encodings", {
   error <- "translating strings with \"bytes\" encoding"
 
@@ -196,29 +202,6 @@ test_that("can compare lists of expressions", {
   expect_equal(vec_equal(x, y), c(TRUE, FALSE))
 })
 
-test_that("equal_scalar() compares", {
-  expect_equal_implemented <- function(x, na) {
-    expect_false(test_equal_scalar(x, 0, x, 1))
-    expect_true(test_equal_scalar(x, 1, x, 1))
-
-    if (!is_null(na)) {
-      expect_false(test_equal_scalar(na, 0, x, 1, na_equal = TRUE))
-      expect_true(is.na(test_equal_scalar(na, 0, x, 1, na_equal = FALSE)))
-    }
-  }
-
-  expect_equal_implemented(c(FALSE, TRUE), NA)
-  expect_equal_implemented(c(0L, 1L), na_int)
-  expect_equal_implemented(c(0, 1), na_dbl)
-  expect_equal_implemented(c(0i, 1i), na_cpl)
-  expect_equal_implemented(c("foo", "bar"), na_chr)
-  expect_equal_implemented(as.raw(c(0, 1)), NULL)
-  expect_equal_implemented(list(0, 1), NULL)
-  expect_equal_implemented(mtcars, vec_init(mtcars))
-
-  expect_error(test_equal_scalar(expression(0), 0, expression(1), 1), "Unsupported")
-})
-
 test_that("vec_equal() silently falls back to base data frame", {
   expect_silent(expect_identical(
     vec_equal(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
@@ -320,6 +303,10 @@ test_that("works recursively with data frame columns", {
   expect_equal(vec_equal_na(df), c(FALSE, FALSE, FALSE, TRUE))
 })
 
+test_that("works with `NULL` input (#1494)", {
+  expect_identical(vec_equal_na(NULL), logical())
+})
+
 test_that("NA propagate symmetrically (#204)", {
   exp <- c(NA, NA)
 
