@@ -1,3 +1,4 @@
+local_name_repair_quiet()
 
 # vec_names() ---------------------------------------------------------
 
@@ -110,7 +111,9 @@ test_that("vec_as_names() repairs names before invoking repair function", {
 })
 
 test_that("vec_as_names() is noisy by default", {
-  verify_output(test_path("output", "test-vec-as-names.txt"), {
+  local_name_repair_verbose()
+
+  expect_snapshot({
     # Noisy name repair
     vec_as_names(c("x", "x"), repair = "unique")
 
@@ -118,7 +121,9 @@ test_that("vec_as_names() is noisy by default", {
     vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)
 
     # Hint at repair argument, if known
-    vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
+    (expect_error(
+      vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
+    ))
   })
 })
 
@@ -157,6 +162,10 @@ test_that("vec_as_names() is quiet when function is supplied (#1018)", {
   )
 })
 
+test_that("vec_as_names() evaluates repair_arg lazily", {
+  expect_silent(vec_as_names(letters, repair_arg = print("oof")))
+})
+
 
 # vec_repair_names() -------------------------------------------------------
 
@@ -424,8 +433,10 @@ test_that("unique-ification has an 'algebraic'-y property", {
 })
 
 test_that("unique_names() and as_unique_names() are verbose or silent", {
-  expect_message(unique_names(1:2), "-> ...1", fixed = TRUE)
-  expect_message(as_unique_names(c("", "")), "-> ...1", fixed = TRUE)
+  local_name_repair_verbose()
+
+  expect_snapshot(unique_names(1:2))
+  expect_snapshot(as_unique_names(c("", "")))
 
   expect_message(regexp = NA, unique_names(1:2, quiet = TRUE))
   expect_message(regexp = NA, as_unique_names(c("", ""), quiet = TRUE))
@@ -512,11 +523,8 @@ test_that("complicated inputs", {
 })
 
 test_that("message", {
-  expect_message(
-    as_universal_names(c("a b", "b c")),
-    "New names:\n* `a b` -> a.b\n* `b c` -> b.c\n",
-    fixed = TRUE
-  )
+  local_name_repair_verbose()
+  expect_snapshot(as_universal_names(c("a b", "b c")))
 })
 
 test_that("quiet", {
@@ -548,17 +556,9 @@ test_that("unnamed input gives uniquely named output", {
 })
 
 test_that("messages by default", {
-  expect_message(
-    vec_repair_names(set_names(1, "a:b"), "universal"),
-    "New names:\n* `a:b` -> a.b\n",
-    fixed = TRUE
-  )
-
-  expect_message(
-    vec_repair_names(set_names(1, "a:b"), ~ make.names(.)),
-    "New names:\n* `a:b` -> a.b\n",
-    fixed = TRUE
-  )
+  local_name_repair_verbose()
+  expect_snapshot(vec_repair_names(set_names(1, "a:b"), "universal"))
+  expect_snapshot(vec_repair_names(set_names(1, "a:b"), ~ make.names(.)))
 })
 
 test_that("quiet = TRUE", {
